from contextlib import ExitStack
from odoo.addons.saas_worker.auth_util import SaasAuth
from odoo.addons.saas_worker.tests.common import SaasWorkerHttpCommon
from odoo.tests import tagged
from unittest.mock import call, patch

@tagged("post_install", "-at_install")
class TestSignup(SaasWorkerHttpCommon):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()

    def test_reset_passwords(self):
        with ExitStack() as stack:
            ResUsers = self.env['res.users']
            stack.enter_context(self._mock_saas_config(from_role="trial"))

            def capture_context(this, *args, **kwargs):
                mock_reset_password.mock_calls[-1].context = this.env.context
            mock_reset_password = stack.enter_context(patch.object(type(ResUsers), 'reset_password', autospec=True,
                                                                   side_effect=capture_context))

            resp = self.url_open("/openerp_enterprise/reset_passwords", json={'params': {'passwords_to_reset': [
                {'login': 'user1@domain.tld', 'saas_trial_reset': '/some/url'},
                {'login': 'user2@domain.tld', 'saas_trial_reset': '/some/other/url'},
            ]}}, auth=SaasAuth())
            resp.raise_for_status()
            self.assertFalse(resp.json().get('error'))

            self.assertEqual(mock_reset_password.mock_calls, [
                call(ResUsers, 'user1@domain.tld'),
                call(ResUsers, 'user2@domain.tld'),
            ])
            self.assertEqual(mock_reset_password.mock_calls[0].context.get('saas_trial_reset'), '/some/url')
            self.assertEqual(mock_reset_password.mock_calls[1].context.get('saas_trial_reset'), '/some/other/url')
